//
//    FILE: unit_test_crc64.cpp
//  AUTHOR: Rob Tillaart
//    DATE: 2021-03-31
// PURPOSE: unit tests for the CRC library
//          https://github.com/RobTillaart/CRC
//          https://github.com/Arduino-CI/arduino_ci/blob/master/REFERENCE.md
//

// supported assertions
// ----------------------------
// assertEqual(expected, actual);               // a == b
// assertNotEqual(unwanted, actual);            // a != b
// assertComparativeEquivalent(expected, actual);    // abs(a - b) == 0 or (!(a > b) && !(a < b))
// assertComparativeNotEquivalent(unwanted, actual); // abs(a - b) > 0  or ((a > b) || (a < b))
// assertLess(upperBound, actual);              // a < b
// assertMore(lowerBound, actual);              // a > b
// assertLessOrEqual(upperBound, actual);       // a <= b
// assertMoreOrEqual(lowerBound, actual);       // a >= b
// assertTrue(actual);
// assertFalse(actual);
// assertNull(actual);

// // special cases for floats
// assertEqualFloat(expected, actual, epsilon);    // fabs(a - b) <= epsilon
// assertNotEqualFloat(unwanted, actual, epsilon); // fabs(a - b) >= epsilon
// assertInfinity(actual);                         // isinf(a)
// assertNotInfinity(actual);                      // !isinf(a)
// assertNAN(arg);                                 // isnan(a)
// assertNotNAN(arg);                              // !isnan(a)


#include <ArduinoUnitTests.h>


#include "Arduino.h"
#include "CRC64.h"


char str[24] = "123456789";
uint8_t * data = (uint8_t *) str;


unittest_setup()
{
}

unittest_teardown()
{
}


unittest(test_crc64_getters)
{
  fprintf(stderr, "TEST CRC64 GETTERS\n");

  CRC64 crc;
  crc.setPolynome(0x04C11DB704C11DB7);
  crc.setStartXOR(0xCE5CA2AD34A16112);
  crc.setEndXOR(0x2AD34A16112CE5CA);
  crc.setReverseIn(false);
  crc.setReverseOut(false);
  
  assertEqual(0x04C11DB704C11DB7, crc.getPolynome());
  assertEqual(0xCE5CA2AD34A16112, crc.getStartXOR());
  assertEqual(0x2AD34A16112CE5CA, crc.getEndXOR());
  assertFalse(crc.getReverseIn());
  assertFalse(crc.getReverseOut());
}


unittest(test_crc64)
{
  fprintf(stderr, "TEST CRC64\n");

  fprintf(stderr, "no reference yet\n");
  assertEqual(1, 1);
  
  // just a dummy test
  CRC64 crc;
  crc.setPolynome(0x04C11DB704C11DB7);
  crc.add(data, 9);
  assertEqual(0xCE5CA2AD34A16112, crc.getCRC());  // 14869938934466568466
}


unittest_main()

// --------
